package project.brain.component;
import java.awt.BorderLayout;
import java.awt.Desktop;
import java.awt.event.*;
import java.net.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.tree.TreePath;

import java.util.*;

import project.brain.openapi.*;
import project.brain.packet.*;
import project.init.ImageButton;
import project.init.Layout;
import project.brain.component.Log;


public class Net_Client implements Runnable {
	private final int DIV_CHAT = 1;
	private final int DIV_OPINION = 2;
	private final int DIV_NODEADD = 3;
	private final int DIV_NODEDEL = 4;
	private final int DIV_CONINFO = 5;
	private final int DIV_SYMPATHYADD = 6;
	private final int DIV_REQ_SYNC = 10;
	private final int DIV_RES_SYNC = 11;
	private final int DIV_SYNC = 12;
	private final int DIV_SYNC_FILEOPENDED = 16;
	private final int DIV_RESET = 13;
	private final int DIV_EXCEED = 14;
	private final int DIV_NODEMODIFY = 15;
	private final int DIV_COMMENT = 20;
	private final int DIV_USERCONNECT = 21;
	private final int DIV_EXIT = 22;
	
	private Socket clientSocket;
	boolean serverUser=false, clientUser=false;
	private Tree tree;
	private String changedNodeName;
	private String beforeNodeName;
	private String userName;
	private String password;
	private boolean flag_ConnectionNetwork = false;
	private ObjectInputStream objIn;
	private ObjectOutputStream objOut;
	private Object rcvObj;
	private Packet packet;

	private ChatFieldAction chatFieldAction;
	private OpinionFieldAction opinionFieldAction;
	private NodeAddAction nodeAddAction;
	private NodeDelAction nodeDelAction;
	private NodeModifyAction nodeModifyAction;
	private SearchAction searchAction;
	private HyperlinkListener hyperlinkListener;
	
	private String attach[] = new String[20]; 
	
	public Net_Client(String ipAddr, int port, String userName, String password, Tree tree, String userType) {
		if(userType.matches("Server"))
			serverUser = true;
		else
			clientUser = true;
		this.password = password;
		init_Network(ipAddr, port, userName);
		this.tree = tree;
		
		chatFieldAction = new ChatFieldAction();
		opinionFieldAction = new OpinionFieldAction();
		nodeAddAction = new NodeAddAction();
		nodeDelAction = new NodeDelAction();
		nodeModifyAction = new NodeModifyAction();
		searchAction = new SearchAction();
		hyperlinkListener = new HyperlinkListener();
		
		Layout.chatInput.addActionListener(chatFieldAction);
		Layout.opinionInputButton.addActionListener(opinionFieldAction);
		Tree.buttonAdd.addActionListener(nodeAddAction);
		Tree.buttonDel.addActionListener(nodeDelAction);
		this.tree.getTreeStructure().addKeyListener(nodeModifyAction);
		Layout.webSearchButton.addActionListener(searchAction);
		Layout.opinionOutputArea.addHyperlinkListener(hyperlinkListener);
	}
	
	public boolean getConnectFlag(){
		return flag_ConnectionNetwork;
	}
	
	public String getUserType(){
		return serverUser ? "Server" : "Client";
	}

	public void init_Network(String ipAddr, int port, String userName) {
		try {
			clientSocket = new Socket(ipAddr, port);
			this.userName = userName;
            
			objOut = new ObjectOutputStream(clientSocket.getOutputStream());
			objIn = new ObjectInputStream(clientSocket.getInputStream());

			Thread thread = new Thread(this);
			thread.start();
            
			UserConnectPacket userConnectPacket = new UserConnectPacket(DIV_USERCONNECT);
			userConnectPacket.setUserName(userName);
			userConnectPacket.setPassword(password);
			objOut.writeObject(userConnectPacket);
			objOut.flush();
			
			flag_ConnectionNetwork = true;
		} catch (IOException e) {
			flag_ConnectionNetwork = false;
			JOptionPane.showConfirmDialog(new JFrame(), e, "Ʈũ  ",
					JOptionPane.DEFAULT_OPTION);
		}
	}

	public void openedFile() throws IOException{
		
		TreeInfoListPacket treeInfoList = new TreeInfoListPacket(DIV_SYNC_FILEOPENDED);
		treeInfoList.setTreeInfoList(tree.getOpinionsPerNodeList());
		objOut.reset();
		objOut.writeObject(treeInfoList);
		objOut.flush();
	}
	
	public void run() {
		try {
			while (!Thread.interrupted()) {
			
				rcvObj = objIn.readObject();
				packet = (Packet) rcvObj;

				if (packet.getHeader() == DIV_CHAT) {
					MessagePacket chatPacket = (MessagePacket) rcvObj;
					Layout.chatOutput.append(chatPacket.getMessage()+"\n");
					Layout.chatScroll.getVerticalScrollBar().setValue(Layout.chatScroll.getVerticalScrollBar().getMaximum());
				}
				else if (packet.getHeader() == DIV_OPINION) {
					OpinionPacket opinionPacket = (OpinionPacket)rcvObj; 
					int index = opinionPacket.getIndex();
					int position = opinionPacket.getPosition();
					String userName = opinionPacket.getUserName();
					String senderCurrentNode = opinionPacket.getNodeName();
					String receiverCurrentNode="";
					try{
						receiverCurrentNode = (String) tree.getCurrentNode().getUserObject(); 
					}catch(NullPointerException npe){
					}
					String msg = opinionPacket.getMessage();

					ArrayList<OpinionsPerNode> opinionList = tree.getOpinionsPerNodeList();
					if(userName.equals(this.userName)){
						opinionList.get(index).setMyOpinionList(userName, msg);
					}
					else{ 
						opinionList.get(index).setOtherOpinionList(userName, position, msg);
					}
					if (senderCurrentNode.matches(receiverCurrentNode)) {
						OpinionsPerNode.viewAllOpinionPerNode(opinionList, index);
					}
					else {
						
					}
					
					Layout.outputScroll.getVerticalScrollBar().setValue(Layout.outputScroll.getVerticalScrollBar().getMaximum());
					Layout.statusLabel.setText("["+userName+"]"+" "+"\""+senderCurrentNode+"\""+"忡 ǰ ߰Ͽϴ.");
					Log.appendLog("["+userName+"]"+" "+"\""+senderCurrentNode+"\""+"忡 ǰ ߰Ͽϴ.");
				}
				else if(packet.getHeader() == DIV_SYMPATHYADD){
					SympathyAddPacket sympathyAddPacket = (SympathyAddPacket)rcvObj;
					ArrayList<OpinionsPerNode> opinionList = tree.getOpinionsPerNodeList();
					int index = sympathyAddPacket.getIndex();
					String userName = sympathyAddPacket.getUserName();
					int position = sympathyAddPacket.getPosition();
					String supporter = sympathyAddPacket.getSupporter();
					if(this.userName.equals(userName)){
						opinionList.get(index).setMyOpinionSympathy(position, supporter);
					}
					else{ 
						opinionList.get(index).setOtherOpinionSympathy(userName, position, supporter);
					}
					OpinionsPerNode.viewAllOpinionPerNode(opinionList, index);
					
					Layout.statusLabel.setText("["+supporter+"]"+" ["+userName+"] ǰ߿ Ͽϴ.");
					Log.appendLog("["+supporter+"]"+" ["+userName+"] ǰ߿ Ͽϴ.");
				}
				else if(packet.getHeader() == DIV_COMMENT){
					CommentPacket commentPacket = (CommentPacket)rcvObj;
					int index = commentPacket.getIndex();
					String userName = commentPacket.getUserName();
					int position = commentPacket.getPosition();
					String writer = commentPacket.getWriter();
					String content = commentPacket.getContent();
					
					ArrayList<OpinionsPerNode> opinionList = tree.getOpinionsPerNodeList();
					if(this.userName.equals(userName)){
						opinionList.get(index).setMyOpinionComment(position, writer, content);
					}
					else{
						opinionList.get(index).setOtherOpinionComment(userName, position, writer, content);
					} 
					OpinionsPerNode.viewAllOpinionPerNode(opinionList, index);
					
					Layout.statusLabel.setText("["+writer+"]"+" ["+userName+"] ǰ߿  ߰Ͽϴ.");
					Log.appendLog("["+writer+"]"+" ["+userName+"] ǰ߿  ߰Ͽϴ.");
				}
				
				else if (packet.getHeader() == DIV_NODEADD) {
					NodeAddPacket nodeAddPacket = (NodeAddPacket)rcvObj;
					String userName = nodeAddPacket.getUserName();

					String parentNodeName = nodeAddPacket.getParentNodeName();
					String newNodeName = nodeAddPacket.getNewNodeName();
					
					tree.automaticAddNode(newNodeName, parentNodeName);
					Layout.statusLabel.setText("["+userName+"]"+" "+"\""+newNodeName+"\""+"带 ߰Ͽϴ.");
					Log.appendLog("["+userName+"]"+" "+"\""+newNodeName+"\""+"带 ߰Ͽϴ.");
				}

				else if (packet.getHeader() == DIV_NODEDEL) {
					NodeDelPacket nodeDelPacket = (NodeDelPacket)rcvObj;
					String userName = nodeDelPacket.getUserName();
					String deletedNodeName = nodeDelPacket.getNodeName();

					tree.automaticDelNode(deletedNodeName);
					Layout.statusLabel.setText("["+userName+"]"+" "+"\""+deletedNodeName+"\""+"带 Ͽϴ.");
					Log.appendLog("["+userName+"]"+" "+"\""+deletedNodeName+"\""+"带 Ͽϴ.");
				}
				

				else if(packet.getHeader() == DIV_NODEMODIFY){
					NodeModifyPacket nodeModifyPacket = (NodeModifyPacket)rcvObj;
					String userName = nodeModifyPacket.getUserName();
					beforeNodeName = nodeModifyPacket.getBeforeNodeName();
					changedNodeName = nodeModifyPacket.getChangedNodeName();

					tree.automaticModifyNode(beforeNodeName, changedNodeName);
					Layout.statusLabel.setText("["+userName+"]"+" "+"\""+beforeNodeName +"带 " +changedNodeName+"\""+"̸ Ͽϴ.");
					Log.appendLog("["+userName+"]"+" "+"\""+beforeNodeName +"带 " +changedNodeName+"\""+"̸ Ͽϴ.");
				}
				
				else if(packet.getHeader() == DIV_CONINFO){
					MessagePacket conInfoPacket = (MessagePacket)rcvObj;
					
					String msg = "̸              IPּ              ӽð\n"+conInfoPacket.getMessage();
					JOptionPane.showMessageDialog(new JFrame(), msg, userName +" Connection-List",
							JOptionPane.INFORMATION_MESSAGE);
				}
				
				else if(packet.getHeader() == DIV_REQ_SYNC){
					TreeInfoListPacket treeInfoList = new TreeInfoListPacket(DIV_RES_SYNC);
					treeInfoList.setTreeInfoList(tree.getOpinionsPerNodeList());
					
					objOut.reset();
					objOut.writeObject(treeInfoList);
					objOut.flush();
				}
				else if(packet.getHeader() == DIV_SYNC){
					try{
						TreeInfoListPacket obj = (TreeInfoListPacket) rcvObj;
						ArrayList<OpinionsPerNode> tmpList = obj.getTreeInfoList();
			            for(int i=1; i<tmpList.size(); i++){
			            	for(int j=0; j<tmpList.get(i).getMyOpinionList().size();){
			            		tmpList.get(i).setOtherOpinionAndSympathyInfo(
			            				tmpList.get(i).getMyOpinionList().get(j).getUserName(),
			            				tmpList.get(i).getMyOpinionList().get(j).getOpinion(), 
			            				tmpList.get(i).getMyOpinionList().get(j).getCntSympathy(), 
			            				tmpList.get(i).getMyOpinionList().get(j).getSupporter(),
			            				tmpList.get(i).getMyOpinionList().get(j).getCommentList());
			            		tmpList.get(i).getMyOpinionList().remove(0);
			            	}
			            }
			            if(tree.getOpinionsPerNodeList().size() > 1)
			            	tree.removeAllNode();
			            
			            tree.setOpinionsPerNodeList(tmpList);
						tree.resetTree();
					}catch(Exception sce){
						sce.printStackTrace();
						JOptionPane.showConfirmDialog(new JFrame(), "ȸǰ ̹ Դϴ.\nȭ ߽ϴ. \n߿ ٽ  ּ.",
								"Sync Error", JOptionPane.DEFAULT_OPTION);
						MessagePacket exitPacket = new MessagePacket(DIV_EXIT);
						exitPacket.setMessage(userName);
						objOut.writeObject(exitPacket);
						objOut.flush();
						closeSocket();
						System.exit(0);					
					}
				}
				
				else if(packet.getHeader() == DIV_RESET){
					
					JOptionPane.showConfirmDialog(new JFrame(), "н尡 ƲȰų,  ̸ ڰ Դϴ. \nȮ ,  ּ.",
							" ", JOptionPane.DEFAULT_OPTION);
					Layout.setNetworkMenuEnabled(false);
					closeSocket();
					return;
				}
				
				else if(packet.getHeader() == DIV_EXCEED){
					
					JOptionPane.showConfirmDialog(new JFrame(), "  ο ʰϿϴ.",
							"ο ʰ", JOptionPane.DEFAULT_OPTION);
					closeSocket();
					return;
				}
			} 
		}catch (Exception ioe) {
			System.out.println(ioe.getMessage());
			Thread.interrupted();
		}
	}

	public void requestConnectionInfo(){
		try {
			MessagePacket conInfoPacket = new MessagePacket(DIV_CONINFO);
			conInfoPacket.setMessage(userName);
			objOut.writeObject(conInfoPacket);
			objOut.flush();
		} catch (Exception e) {}
	}
	
	public void noticeExit(){
		try{
			MessagePacket exitPacket = new MessagePacket(DIV_EXIT);
			exitPacket.setMessage(userName);
			objOut.writeObject(exitPacket);
			objOut.flush();
		}catch(Exception e){}
	}
	
	public void closeSocket(){
		noticeExit();
		try {
			flag_ConnectionNetwork = false;
			Layout.chatInput.removeActionListener(chatFieldAction);
			Layout.opinionInputButton.removeActionListener(opinionFieldAction);
			Tree.buttonAdd.removeActionListener(nodeAddAction);
			Tree.buttonDel.removeActionListener(nodeDelAction);
			Layout.webSearchButton.removeActionListener(searchAction);
			Layout.opinionOutputArea.removeHyperlinkListener(hyperlinkListener);
			clientSocket.close();			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	class ChatFieldAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			try {
				String msg = Layout.chatInput.getText();
				
				if(msg.trim().equals(""))
					return;
				
				MessagePacket chatPacket = new MessagePacket(DIV_CHAT);
				chatPacket.setMessage("["+userName+"]" + msg);
				objOut.writeObject(chatPacket);
				objOut.flush();
				Layout.chatInput.setText("");

			} catch (IOException ie) {
				if(clientSocket.isClosed()){
					JOptionPane.showConfirmDialog(new JFrame(), "Ͽ Ǿϴ.\nٽ  õ .\n"+ie,
							" ", JOptionPane.DEFAULT_OPTION);
					Layout.setNetworkMenuEnabled(false);
				}
			}
		}
	}

	class OpinionFieldAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			try {
				String msg = Layout.opinionInputArea.getText().replace("\n", "<br>");
				
				if(msg.trim().equals(""))
					return;
				
				String nodeName = (String) tree.getCurrentNode().getUserObject();
				
				if(nodeName.equals(tree.getOpinionsPerNodeList().get(0).getNodeName())){
					JOptionPane.showConfirmDialog(new JFrame(), "ֻ  극ν  Ȯ   ,\nǰ Է  ϴ.",
							"Է ", JOptionPane.DEFAULT_OPTION);
					return;
				}
				
				ArrayList<OpinionsPerNode> opinionList = tree.getOpinionsPerNodeList();
				int index=0;
				for (int i = 0; i < opinionList.size(); i++){
					try{
					if(nodeName.equals(opinionList.get(i).getNodeName())){
						index=i;
						break;
					}
					}catch(NullPointerException npe){}
				}
				int position = opinionList.get(index).getMyOpinionListSize();
				
				msg = "<font size=4>" +msg+"</font>";
				msg+= "<font size=3 face= color=gray><a href= http:#"+index +"&"+userName+"&"+position+">[]</a></font>"+
				"<font size=3 face= color=gray><a href= http:#"+index +"&"+userName+"&"+position+">[]</a></font>";
				OpinionPacket opinionPacket = new OpinionPacket(DIV_OPINION);
				opinionPacket.setIndex(index);
				opinionPacket.setUserName(userName);
				opinionPacket.setPosition(position);
				opinionPacket.setMessage(msg);
				opinionPacket.setNodeName(nodeName);
				
				objOut.writeObject(opinionPacket);
				objOut.flush();

				Layout.opinionInputArea.setText("");

			} catch (Exception ie) {
				if(clientSocket.isClosed()){
					JOptionPane.showConfirmDialog(new JFrame(), "Ͽ Ǿϴ.\nٽ  õ .\n"+ie,
							" ", JOptionPane.DEFAULT_OPTION);
					Layout.setNetworkMenuEnabled(false);
				}
				else{
				JOptionPane.showConfirmDialog(new JFrame(), "尡 õ ʾҽϴ.\n"+ie,
						"ǰ ", JOptionPane.DEFAULT_OPTION);
				}
			}
		}
	}

	class NodeAddAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			TreePath treePath = tree.getTreeStructure().getSelectionPath();

			if (treePath == null) {
				JOptionPane.showConfirmDialog(new JFrame(), "带  ּ.", " ", JOptionPane.DEFAULT_OPTION);
				return;
			}
			
			String newNodeName = tree.getNodeNameField().getText().trim();
			if(newNodeName.equals("")){
				JOptionPane.showConfirmDialog(new JFrame(), " ̸ Էּ.", "߰",
						JOptionPane.DEFAULT_OPTION);
				return;
			}
			for(int i=0; i<tree.getOpinionsPerNodeList().size(); i++){
				if(newNodeName.equals(tree.getOpinionsPerNodeList().get(i).getNodeName())){
					JOptionPane.showConfirmDialog(new JFrame(), " ̸ 尡 ̹ մϴ.",
							"߰", JOptionPane.DEFAULT_OPTION);
					tree.getNodeNameField().setText("");
					return;
				}
			}

			tree.setTreePath(treePath);
			try {
				String parentNodeName = (String) tree.getCurrentNode().getUserObject(); 
				NodeAddPacket nodeAddPacket = new NodeAddPacket(DIV_NODEADD);
				nodeAddPacket.setUserName(userName);
				nodeAddPacket.setNewNodeName(newNodeName);
				nodeAddPacket.setParentNodeName(parentNodeName);

				objOut.writeObject(nodeAddPacket);
				objOut.flush();
				tree.getNodeNameField().setText("");

			} catch (Exception ie) {
				if(clientSocket.isClosed()){
					JOptionPane.showConfirmDialog(new JFrame(), "Ͽ Ǿϴ.\nٽ  õ .\n"+ie,
							" ", JOptionPane.DEFAULT_OPTION);
					Layout.setNetworkMenuEnabled(false);
				}
			}
		}
	}

	class NodeDelAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if(tree.getCurrentNode().isRoot()){
				JOptionPane.showConfirmDialog(new JFrame(), "ֻ    ϴ. \n",
						" ", JOptionPane.DEFAULT_OPTION);
				return;
			}
			try {
				String nodeName = tree.getCurrentNode().getUserObject().toString();
				NodeDelPacket nodeDelPacket = new NodeDelPacket(DIV_NODEDEL);
				nodeDelPacket.setUserName(userName);
				nodeDelPacket.setNodeName(nodeName);
				
				objOut.writeObject(nodeDelPacket);
				objOut.flush();
			} catch (Exception ie) {
				if(clientSocket.isClosed()){
					JOptionPane.showConfirmDialog(new JFrame(), "Ͽ Ǿϴ.\nٽ  õ .\n"+ie,
							" ", JOptionPane.DEFAULT_OPTION);
					Layout.setNetworkMenuEnabled(false);
				}
			}
		}
	}
	
	class NodeModifyAction implements KeyListener {
		@Override
		public void keyReleased(KeyEvent e) {
			try{
				if(e.getKeyCode() == KeyEvent.VK_F2){
					if(tree.getOpinionsPerNodeList().get(0).getNodeName().equals((String)tree.getCurrentNode().getUserObject())){
						JOptionPane.showConfirmDialog(new JFrame(), "ֻ  ̸   ϴ.",
								"̸  ", JOptionPane.DEFAULT_OPTION);
						return;
					}
					tree.getTreeStructure().setEditable(true);
				}
				if(e.getKeyChar() == KeyEvent.VK_ENTER){
					changedNodeName = (String) tree.getCurrentNode().getUserObject();
					beforeNodeName = tree.getBeforeNodeName();
					for(int i=0; i<tree.getOpinionsPerNodeList().size(); i++){
						if(changedNodeName.equals(tree.getOpinionsPerNodeList().get(i).getNodeName())){
							JOptionPane.showConfirmDialog(new JFrame(), " ̸ 尡 ̹ մϴ.",
									"̸  ", JOptionPane.DEFAULT_OPTION);
							tree.getCurrentNode().setUserObject(beforeNodeName);
							tree.automaticModifyNode(beforeNodeName, beforeNodeName);
							return;
						}
					}
					if(!(beforeNodeName.equals(changedNodeName))){
						NodeModifyPacket nodeModifyPacket = new NodeModifyPacket(DIV_NODEMODIFY);
						nodeModifyPacket.setUserName(userName);
						nodeModifyPacket.setBeforeNodeName(beforeNodeName);
						nodeModifyPacket.setChangedNodeName(changedNodeName);

						objOut.writeObject(nodeModifyPacket);
						objOut.flush();
					}
				}
			}catch(Exception ie){
				JOptionPane.showConfirmDialog(new JFrame(), "Ͽ Ǿϴ.\nٽ  õ .\n"+ie,
						"̸  ", JOptionPane.DEFAULT_OPTION);
			}
		}
		public void keyTyped(KeyEvent e) {}
		public void keyPressed(KeyEvent e) {}
	}
	
	
	class SearchAction implements ActionListener {
		private JFrame webFrame;
		private JPanel searchPanel;
		private JPanel iconPanel = new JPanel(new BorderLayout());
		private JEditorPane resultPanel = new JEditorPane();
		
		private JComboBox categoryCombo = new JComboBox();
		private int categoryNumber=1;
		private JTextField keywordField = new JTextField(15);
		private JButton searchButton = new JButton("Search");
		private DaumParse daum = new DaumParse();
		private int pageNo = 1;
		private ImageButton prevButton = new ImageButton("images/prev_button.png");
		private ImageButton nextButton = new ImageButton("images/next_button.png");
		private JLabel pageLabel = new JLabel();
		private Category[] result;
		

		public void attachResult(){
			if(result.length==0 || result == null){
				JOptionPane.showConfirmDialog(new JFrame(), "˻ ϴ.",
						"˻", JOptionPane.DEFAULT_OPTION);
				resultPanel.setText("");
				webFrame.setSize(400, 100);
				webFrame.remove(iconPanel);
				return;
			}
			webFrame.setSize(400, 600);
			String output="";
			
			switch(categoryNumber){
			case 1:
				DaumBlogSearchResult blogResult;
				for(int i=0; i<result.length; i++){
					blogResult = (DaumBlogSearchResult)result[i];
					attach[i]= "<a href="+blogResult.getLink()+">"+blogResult.getTitle()+"</a> <br/>("+
					blogResult.getPubDate()+"/"+blogResult.getAuthor()+")<br/>"+blogResult.getDescription()+"<br/><br/>";
					
					output += "<a alt=\"ǰ߿ ߰ϱ\" href=http:˻̱#"+i+"><img src=http://cdn.iconfinder.net/data/icons/fatcow/32x32_0140/bullet_add.png border=\"0\"/></a>  "
					+ attach[i];
				}
				resultPanel.setText(output);
				break;
			case 2:
				DaumImageSearchResult imageResult;
				for(int i=0; i<result.length; i++){
					imageResult = (DaumImageSearchResult)result[i];
					attach[i]= "<a href="+imageResult.getLink()+">"+imageResult.getTitle()+"</a> <br/>("+
					imageResult.getPubDate()+"/"+imageResult.getAuthor()+")<br/>"+
					"<img src="+imageResult.getThumbnail()+"/><br/><br/>";
					
					output += "<a alt=\"ǰ߿ ߰ϱ\" href=http:˻̱#"+i+"><img src=http://cdn.iconfinder.net/data/icons/fatcow/32x32_0140/bullet_add.png border=\"0\"/></a>  "
					+ attach[i];
				}
				resultPanel.setText(output);
				break;
			case 3:
				DaumKnowledgeSearchResult knowResult;
				for(int i=0; i<result.length; i++){
					knowResult = (DaumKnowledgeSearchResult)result[i];
					attach[i] = "<a href="+knowResult.getLink()+">"+knowResult.getTitle()+"</a> <br/>("+
					knowResult.getPubDate()+"/"+knowResult.getAuthor()+")<br/>" + knowResult.getDescription()+"<br/><br/>";
					
					output += "<a alt=\"ǰ߿ ߰ϱ\" href=http:˻̱#"+i+"><img src=http://cdn.iconfinder.net/data/icons/fatcow/32x32_0140/bullet_add.png border=\"0\"/></a>  "
					+ attach[i];
				}
				resultPanel.setText(output);
				break;
			case 4:
				DaumBookSearchResult bookResult;
				for(int i=0; i<result.length; i++){
					bookResult = (DaumBookSearchResult)result[i];
					attach[i] = "<a href="+bookResult.getLink()+">"+bookResult.getTitle()+"</a> <br/>("+
					bookResult.getPubDate()+"/"+bookResult.getAuthor()+"/"+bookResult.getPubCompany()+
					"/"+bookResult.getCategory()+"/"+bookResult.getSalePrice()+")<br/>"+
					"<img src=\""+bookResult.getCoverUrl()+"\"/>"+bookResult.getDescription()+"<br/><br/>";
					
					output += "<a alt=\"ǰ߿ ߰ϱ\" href=http:˻̱#"+i+"><img src=http://cdn.iconfinder.net/data/icons/fatcow/32x32_0140/bullet_add.png border=\"0\"/></a>  "
					+ attach[i];
				}
				resultPanel.setText(output);
				break;
			}
			
			webFrame.add(iconPanel, BorderLayout.SOUTH);
		}
		
		public void actionPerformed(ActionEvent e) {
			if(webFrame != null){//˻
				webFrame.setVisible(true);
				return;
			}
			
			webFrame = new JFrame("˻");
			webFrame.setLayout(new BorderLayout());
			
			keywordField.setText("");
			keywordField.setFocusable(true);
			
			categoryCombo.removeAllItems();
			categoryCombo.addItem("1.α");
			categoryCombo.addItem("2.̹");
			categoryCombo.addItem("3.");
			categoryCombo.addItem("4.å");
			
			resultPanel.setEditable(false);
			resultPanel.setText("");
			resultPanel.setContentType("text/html");
			resultPanel.addHyperlinkListener(new HyperlinkListener());
			
			categoryCombo.addItemListener(new ItemListener() {
				@Override
				public void itemStateChanged(ItemEvent e) {
					if(e.getStateChange() == ItemEvent.SELECTED){
						categoryNumber = Integer.parseInt(e.getItem().toString().substring(0, e.getItem().toString().indexOf('.')));
					}
				}
			});

			searchPanel = new JPanel(new BorderLayout());
			searchPanel.add(categoryCombo, BorderLayout.WEST);
			searchPanel.add(keywordField, BorderLayout.CENTER);
			searchPanel.add(searchButton, BorderLayout.EAST);
			
			webFrame.add(searchPanel, BorderLayout.NORTH);
			webFrame.add(new JScrollPane(resultPanel), BorderLayout.CENTER);
			webFrame.setSize(400, 100);
			webFrame.setVisible(true);
			
			nextButton.setToolTipText(" ");
			prevButton.setToolTipText(" ");
			pageLabel.setHorizontalAlignment(SwingConstants.CENTER);
			iconPanel.add(prevButton, BorderLayout.WEST);
			iconPanel.add(pageLabel, BorderLayout.CENTER);
			iconPanel.add(nextButton, BorderLayout.EAST);
			
			searchButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					try{
						pageNo = 1;
						pageLabel.setText("1");
						result = daum.parse(categoryNumber, keywordField.getText(), pageNo);
						attachResult();
					}catch(Exception e){
						e.printStackTrace();
					}
				}
			});
			nextButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent arg1){
					pageNo++;
					result = daum.parse(categoryNumber, keywordField.getText(), pageNo);
					if(result.length != 0){
						attachResult();
						pageLabel.setText(pageNo+"");
					}
					else{
						pageNo--;
						JOptionPane.showConfirmDialog(new JFrame(), " ˻ ϴ.",
								"˻", JOptionPane.DEFAULT_OPTION);
					}
				}
			});
			prevButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent arg2){
					if(pageNo > 1){
						pageNo--;
						pageLabel.setText(pageNo+"");
						result = daum.parse(categoryNumber, keywordField.getText(), pageNo);
						attachResult();
					}else{
						JOptionPane.showConfirmDialog(new JFrame(), " ˻ ϴ.",
								"˻", JOptionPane.DEFAULT_OPTION);
					}
				}
			});
		}
	}
	
	
	public class HyperlinkListener implements javax.swing.event.HyperlinkListener {
 
		@Override
		public void hyperlinkUpdate(HyperlinkEvent e) {
			try{
				if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
				{ 
					String url = e.getURL().toString();
					final String data[] = url.substring(5, url.length()).split("#");
					if (data[0].equals("")){			
						ArrayList<OpinionsPerNode> opinionList = tree.getOpinionsPerNodeList();
						String tmpData[] = data[1].split("&");
						boolean duplFlag=true;
						if(tmpData[1].equals(userName)){
							if(opinionList.get(Integer.parseInt(tmpData[0])).chkDuplMyOpinionSympathy(Integer.parseInt(tmpData[2]), userName))
								duplFlag=false;
						}
						else{
							if(opinionList.get(Integer.parseInt(tmpData[0])).chkDuplOtherOpinionSympathy(tmpData[1], Integer.parseInt(tmpData[2]), userName))
								duplFlag=false;
						}

						if(!duplFlag){	
							SympathyAddPacket sympathyAddPacket = new SympathyAddPacket(DIV_SYMPATHYADD);
							sympathyAddPacket.setIndex(Integer.parseInt(tmpData[0]));
							sympathyAddPacket.setUserName(tmpData[1]);
							sympathyAddPacket.setPosition(Integer.parseInt(tmpData[2]));
							sympathyAddPacket.setSupporter(userName);

							objOut.writeObject(sympathyAddPacket);
							objOut.flush();
						}else
							JOptionPane.showConfirmDialog(new JFrame(), "̹ ϼ̽ϴ",	" ", JOptionPane.DEFAULT_OPTION);						
					}	
					
					else if(data[0].equals("")){
						new CommentAction(data[1]);
					}
					
					else if(data[0].equals("˻̱")){
						String tmp = "<DIV style=\"BORDER-BOTTOM: #e7fdb5 1px solid; BORDER-LEFT: #e7fdb5 1px solid; PADDING-BOTTOM: 10px; BACKGROUND-COLOR: #e7fdb5; PADDING-LEFT: 10px; PADDING-RIGHT: 10px; BORDER-TOP: #e7fdb5 1px solid; BORDER-RIGHT: #e7fdb5 1px solid; PADDING-TOP: 10px\" class=txc-textbox>"+
						attach[Integer.parseInt(data[1])]+"</DIV>";
						
						Layout.opinionInputArea.setText(Layout.opinionInputArea.getText() + tmp);
						JOptionPane.showConfirmDialog(new JFrame(), "ǰԷâ ˻ ߰Ǿϴ.",	"˻ ߰", JOptionPane.DEFAULT_OPTION);
					}
					
					else{
						Desktop.getDesktop().browse(new URI(url));
					}
				}
			} catch (Exception ie) {
				if(clientSocket.isClosed()){
					JOptionPane.showConfirmDialog(new JFrame(), "Ͽ Ǿϴ.\nٽ  õ .\n"+ie,
							" ", JOptionPane.DEFAULT_OPTION);
					Layout.setNetworkMenuEnabled(false);
				}
			}
		}
	}
	
	class CommentAction{
		JFrame netFrame= new JFrame("Է");
		JLabel commentLabel = new JLabel("");
		JTextField commentField = new JTextField(30);
		JButton inputButton = new JButton("Է");
		public CommentAction(String data){
			final String tmpData[] = data.split("&");
			new BorderLayout();
			netFrame.add(commentLabel, BorderLayout.WEST);
			new BorderLayout();
			netFrame.add(commentField, BorderLayout.CENTER);
			new BorderLayout();
			netFrame.add(inputButton, BorderLayout.EAST);
			netFrame.setSize(350, 60);
			netFrame.setVisible(true);
			inputButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					String comment = commentField.getText().trim();
					CommentPacket commentPacket = new CommentPacket(DIV_COMMENT);
					commentPacket.setIndex(Integer.parseInt(tmpData[0]));
					commentPacket.setUserName(tmpData[1]);
					commentPacket.setPosition(Integer.parseInt(tmpData[2]));
					commentPacket.setWriter(userName);
					commentPacket.setContent(comment);
					
					try {
						objOut.writeObject(commentPacket);
						objOut.flush();
					} catch (IOException e) {
						e.printStackTrace();
					}
					
					netFrame.setVisible(false);
				}
			});
		}
	}
}
