package project.brain.component;
import java.io.*;
import java.net.*;
import java.text.SimpleDateFormat;
import java.util.*;
import project.brain.packet.*;

public class Net_Handler implements Runnable {
	private static Vector<Net_Handler> connectionList = new Vector<Net_Handler>(); 
	private static boolean bStartMeeting=false;
	private Socket client;
	private String clientName;
	private Date connectTime;
	private ObjectInputStream objIn;
	private ObjectOutputStream objOut;
	private Thread listener;
	private final int DIV_CHAT = 1;
	private final int DIV_OPINION = 2;
	private final int DIV_NODEADD = 3;
	private final int DIV_CONINFO = 5;
	private final int DIV_REQ_SYNC = 10;
	private final int DIV_RES_SYNC = 11;
	private final int DIV_SYNC = 12;
	private final int DIV_RESET = 13;
	private final int DIV_NODEMODIFY = 15;
	private final int DIV_SYNC_FILEOPENDED = 16;
	private final int DIV_USERCONNECT = 21;
	private final int DIV_EXIT = 22;
	private long startTime;
	

	public Net_Handler(Socket client) {
		this.client = client;
		try {
			objOut = new ObjectOutputStream(client.getOutputStream());
			objIn = new ObjectInputStream(client.getInputStream());
			
			listener = new Thread(this);
			listener.start();
		} catch (Exception ioe) {
		} 
	}
	
	private Object rcvObj;
	private Packet packet;
	public void run() {
		try {
			while (!Thread.interrupted()) {
				rcvObj = objIn.readObject();
				packet = (Packet)rcvObj;
				if (packet.getHeader() == DIV_USERCONNECT) {	
					UserConnectPacket userConnectPacket = (UserConnectPacket)rcvObj;
					String password;
					try{
						password = userConnectPacket.getPassword();
					}catch(IndexOutOfBoundsException e){
						password="";
					}
					if(!Net_Server.getPassword().isEmpty() && !password.equals(Net_Server.getPassword())){
						Packet resetPacket = new Packet(DIV_RESET);
						objOut.writeObject(resetPacket);
						objOut.flush();
						return;
					}
					clientName = userConnectPacket.getUserName();
					Net_Handler handler;
					synchronized (connectionList) {
						Enumeration<Net_Handler> e = connectionList.elements();
						while (e.hasMoreElements()) {
							handler = e.nextElement();
	
							if(handler.clientName.equals(clientName)){
								Packet resetPacket = new Packet(DIV_RESET);
								objOut.writeObject(resetPacket);
								objOut.flush();
								return;
							}
						}
					}
					MessagePacket chatPacket = new MessagePacket(DIV_CHAT);
					chatPacket.setMessage("[" + clientName + "]" + " ϼ̽ϴ.");
					rcvObj = chatPacket;
					Log.appendLog("[" + clientName + "]" + " ϼ̽ϴ.");

					Calendar calendar = Calendar.getInstance();
					connectTime = calendar.getTime();
					connectionList.addElement(this);
					
					if(bStartMeeting){
						Net_Handler serverClient = connectionList.firstElement();
						try {
							Packet reqSyncPacket = new Packet(DIV_REQ_SYNC);
							serverClient.objOut.writeObject(reqSyncPacket);
							serverClient.objOut.flush();
							
							continue;
						} catch (IOException ex) {
							System.out.println("ȭ û Ŷ  ");
							ex.printStackTrace();
						}
					}
				}
				
				else if (packet.getHeader() == DIV_CONINFO) {
					MessagePacket messagePacket = (MessagePacket) rcvObj;
					Net_Handler handler;
					ArrayList<Net_Handler> nonExistHandler = new ArrayList<Net_Handler>();
					Net_Handler destinationClient = null;
					String connectionInfo="";
					String clientName = messagePacket.getMessage();
					SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd, [HH:mm:ss]", Locale.KOREA);
					synchronized (connectionList) {
						Enumeration<Net_Handler> e = connectionList.elements();
						while (e.hasMoreElements()) {
							handler = e.nextElement();
							
							if(!handler.listener.isAlive()){
								MessagePacket chatPacket = new MessagePacket(DIV_CHAT);
								chatPacket.setMessage(handler.clientName+"  ϼ̽ϴ.");
								broadcast(chatPacket);
								Log.appendLog(handler.clientName+"  ϼ̽ϴ.");
								nonExistHandler.add(handler);
								
								continue;
							}
							connectionInfo = connectionInfo+handler.clientName
							+"  "+handler.client.getInetAddress()+"   "+dateFormat.format(handler.connectTime)+"\n";
							
							if (destinationClient == null && handler.clientName.equals(clientName)) {
								destinationClient = handler;
							}
						}
						for(int i=0; i<nonExistHandler.size(); i++){
							connectionList.removeElement(nonExistHandler.get(i));
						}
						try {
							MessagePacket conInfoPacket = new MessagePacket(DIV_CONINFO);
							conInfoPacket.setMessage(connectionInfo);
							destinationClient.objOut.writeObject(conInfoPacket);
							destinationClient.objOut.flush();
							continue;
						} catch (IOException ex) {
						}
					}
				}
				else if(packet.getHeader() == DIV_EXIT){
					MessagePacket messagePacket = (MessagePacket)rcvObj;
					Net_Handler handler;
					String clientName = messagePacket.getMessage();
					synchronized(connectionList){
						Enumeration<Net_Handler> e = connectionList.elements();
						while(e.hasMoreElements()){
							handler = e.nextElement();
							if(handler.clientName.equals(clientName)){
								connectionList.removeElement(handler);
								break;
							}
						}
					}
					MessagePacket chatPacket = new MessagePacket(DIV_CHAT);
					chatPacket.setMessage(clientName+"  ϼ̽ϴ.");
					rcvObj = chatPacket;
					Net_Server.setCurrentAttendeeNumber(connectionList.size());
					Log.appendLog(clientName+"  ϼ̽ϴ.");
				}
				
				else if(packet.getHeader() == DIV_RES_SYNC){
					TreeInfoListPacket treeInfoListPacket= new TreeInfoListPacket(DIV_SYNC);
					treeInfoListPacket.setTreeInfoList(((TreeInfoListPacket)rcvObj).getTreeInfoList());
					
					try {
						connectionList.lastElement().objOut.writeObject(treeInfoListPacket);
						connectionList.lastElement().objOut.flush();
					} catch (Exception e) {
						System.out.println("ddd");
						e.printStackTrace();
					}
					
					MessagePacket chatPacket = new MessagePacket(DIV_CHAT);
					chatPacket.setMessage("[" + connectionList.lastElement().clientName + "]" + " \"" + ((new Date().getTime()-startTime)/(1000*60)) +"\" ϼ̳׿. \n  ּ!");
					rcvObj = chatPacket;
					Log.appendLog("[" + connectionList.lastElement().clientName + "]" + " \"" + ((new Date().getTime()-startTime)/(1000*60)) +"\" ϼ̽ϴ.");
				}
				else if(packet.getHeader() == DIV_SYNC_FILEOPENDED){
					bStartMeeting = true;
					Net_Handler handler;
					TreeInfoListPacket treeInfoListPacket = new TreeInfoListPacket(DIV_SYNC);
					treeInfoListPacket = (TreeInfoListPacket)rcvObj;
					synchronized (connectionList) {
						Enumeration<Net_Handler> e = connectionList.elements();
						e.nextElement();
						while (e.hasMoreElements()) {
							handler = e.nextElement();
							handler.objOut.writeObject(treeInfoListPacket);
							handler.objOut.flush();
						}
					}
					MessagePacket chatPacket = new MessagePacket(DIV_CHAT);
					chatPacket.setMessage("   Ӱ ȭǾϴ.");
					rcvObj = chatPacket;
					Log.appendLog("   Ӱ ȭǾϴ.");
				}
				
				broadcast(rcvObj);
				if(!bStartMeeting && (packet.getHeader() == DIV_NODEADD) || 
						packet.getHeader() == DIV_NODEMODIFY || packet.getHeader() == DIV_OPINION){
					bStartMeeting = true;
					startTime = new Date().getTime();
				}
			}
		} catch (Exception ioe) {
			System.out.println(ioe.getMessage());
			return;
		} 
	}
	
	public static void setBStartMeeting(){
		bStartMeeting = true;
	}
	public void broadcast(Object obj) {
		Net_Handler handler;
		synchronized (connectionList) {
			Enumeration<Net_Handler> e = connectionList.elements();
			while (e.hasMoreElements()) {
				handler = e.nextElement();
				try {
					handler.objOut.writeObject(obj);
					handler.objOut.flush();
				} catch (IOException ex) {
				}
			}
		}
	}
}
