package project.brain.component;
import java.awt.Color;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.util.*;
import java.io.*;

public class Tree extends JFrame implements TreeSelectionListener, Serializable {

	private static final long serialVersionUID = 1L;
	public final static String NODE_ADD = "߰";
	public final static String NODE_DEL = "";

	private JTree treeStructure;
	public Box toolBar = Box.createHorizontalBox();
	private JTextField nodeNameField;
	static JButton buttonAdd, buttonDel;
	private DefaultMutableTreeNode currentNode;
	private ArrayList<OpinionsPerNode> opinionsPerNodeList = new ArrayList<OpinionsPerNode>();
	private ArrayList<OpinionsPerNode> tempList = new ArrayList<OpinionsPerNode>();
	private TreePath treePath;

	private DefaultMutableTreeNode root;
	private DefaultTreeModel treeModel;
	private String beforeNodeName;
	
	public ArrayList<OpinionsPerNode> getOpinionsPerNodeList() {
		return opinionsPerNodeList;
	}

	public Tree() {
		root = new DefaultMutableTreeNode("BrainStorming");

		OpinionsPerNode rootNode = new OpinionsPerNode(root);
		rootNode.setNodeName("BrainStorming");
		opinionsPerNodeList.add(rootNode);
		String intro = "Brain_Storming<br><br>"
				+ "    Ʈũ ̿ ȸȯ ϴ α׷ <br><br>" + "    극ν̶ <br>"
				+ "      : â   ̵ âϱ   ϴ ȸ<br>"
				+ "      <br>" + "          ټ  Ǵ ̵ . <br>"
				+ "        ̵     ̵  ɼ .<br>"
				+ "        Ϲ ̵    .<br>"
				+ "        йϰ ϱ  ǰ  ̵ Ѵ.<br>" + "<br>";

		opinionsPerNodeList.get(0).setMyOpinionList(intro);

		treeStructure = new JTree(root);

		nodeNameField = new JTextField(13);
		toolBar.add(nodeNameField);

		buttonAdd = new JButton(NODE_ADD);
		toolBar.add(buttonAdd);

		buttonDel = new JButton(NODE_DEL);
		toolBar.add(buttonDel);

		treeStructure.addTreeSelectionListener(this);
	}

	@SuppressWarnings("unchecked")
	public void setOpinionsPerNodeList(Object tempList) {
		this.tempList = (ArrayList<OpinionsPerNode>) tempList;
	}

	public void setTreePath(TreePath treePath) {
		this.treePath = treePath;
	}

	public TreePath getTreePath() {
		return treePath;
	}

	public  DefaultMutableTreeNode getCurrentNode() {
		return currentNode;
	}

	public JTextField getNodeNameField() {
		return nodeNameField;
	}

	public JTree getTreeStructure() {
		return treeStructure;
	}

	public DefaultTreeModel getTreeModel() {
		return treeModel;
	}

	public void resetTree() {
		int index = 1;
		while (index != tempList.size()) {
			DefaultMutableTreeNode node = new DefaultMutableTreeNode(tempList
					.get(index).getNodeName());
			OpinionsPerNode opinionsPerNode = new OpinionsPerNode(node);
			opinionsPerNode.setMyOpinionList(tempList.get(index).getMyOpinionList());
			opinionsPerNode.setOtherOpinionList(tempList.get(index).getOtherOpinionList());
			opinionsPerNode.setNodeName(tempList.get(index).getNodeName());
			opinionsPerNodeList.add(opinionsPerNode);

			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) tempList
					.get(index).getNode().getParent();

			if (parentNode.isRoot()) {
				root.add(node);
			}
			else {
				for (int i = 1; i < tempList.size(); i++) {
					if (parentNode.toString().matches(opinionsPerNodeList.get(i).getNodeName())) {
						opinionsPerNodeList.get(i).getNode().add(node);
						treeModel = (DefaultTreeModel) treeStructure.getModel();
						treeModel.reload(opinionsPerNodeList.get(i).getNode());
						treeStructure.expandPath(treePath);
						break;
					}
				}
			}
			index++;
		}
	}

	public void automaticAddNode(String newNodeName, String parentNodeName) {

		DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(newNodeName);
		OpinionsPerNode opinionsPerNode = new OpinionsPerNode(newNode);
		opinionsPerNode.setNodeName(newNodeName);
		opinionsPerNodeList.add(opinionsPerNode);

		for (int i = 0; i < opinionsPerNodeList.size(); i++) {
			if (opinionsPerNodeList.get(i).getNodeName().matches(parentNodeName)) {
				opinionsPerNodeList.get(i).getNode().add(newNode);
				treeModel = (DefaultTreeModel) treeStructure.getModel();
				treeModel.reload(opinionsPerNodeList.get(i).getNode());
				treeStructure.expandPath(treePath);
				break;
			}
		}
	}
	
	public void automaticDelNode(String deletedNodeName){
		DefaultMutableTreeNode node = null;
	    Enumeration<?> e = root.breadthFirstEnumeration();
	    while (e.hasMoreElements()) {
		    node = (DefaultMutableTreeNode) e.nextElement();								  		    
		    if (node.getUserObject().toString().equals(deletedNodeName)) 
		    	break;
	    }

		DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) node
				.getParent();
		treeModel.removeNodeFromParent(node);

		treeModel.reload(parentNode);

		for (int i = 1; i < opinionsPerNodeList.size(); i++) {
			if (opinionsPerNodeList.get(i).getNode().equals(node)) {
				removeNodeInList(node, i);
				break;
			}
		}
	}
	
	public void automaticModifyNode(String beforeNodeName, String changedNodeName){
		int i;
		DefaultMutableTreeNode node = null;
	    Enumeration<?> e = root.breadthFirstEnumeration();
	    this.beforeNodeName = changedNodeName;
	    while (e.hasMoreElements()) {
		    node = (DefaultMutableTreeNode) e.nextElement();
		    if (changedNodeName.equals(node.getUserObject().toString()) || beforeNodeName.equals(node.getUserObject().toString())) {
			  	for (i = 0; i < opinionsPerNodeList.size(); i++) {
					OpinionsPerNode opinionPerNode = (OpinionsPerNode) opinionsPerNodeList.get(i);
					if (opinionPerNode.getNode() == node) {
						opinionPerNode.setNodeName(changedNodeName);
						break;
					}
				}
			  	node.setUserObject(changedNodeName);
			  	
				treeModel = (DefaultTreeModel) treeStructure.getModel();
				treeModel.reload(opinionsPerNodeList.get(i).getNode());
				treeStructure.expandPath(treePath);
			  	break;
		    }
	    }
	}

	public void valueChanged(TreeSelectionEvent ev) {
		currentNode = (DefaultMutableTreeNode) (ev.getPath().getLastPathComponent());
		beforeNodeName = (String)currentNode.getUserObject();
		
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) treeStructure.getCellRenderer();
		renderer.setTextSelectionColor(new Color(204,051,153));
	    renderer.setBackgroundSelectionColor(new Color(255,255,204));
		
		for (int i = 0; i < opinionsPerNodeList.size(); i++) {
			OpinionsPerNode currentOpinionPerNode = (OpinionsPerNode) opinionsPerNodeList.get(i);
			if (currentOpinionPerNode.getNode() == currentNode) {
				OpinionsPerNode.viewAllOpinionPerNode(opinionsPerNodeList, i);
			
				break;
			}
		}
		treeStructure.setEditable(false);
	}
	
	public String getBeforeNodeName(){
		return beforeNodeName;
	}

	public void removeNodeInList(DefaultMutableTreeNode parentNode,
			int parent_Pos) {
		DefaultMutableTreeNode childNode;
		for (int child_Pos = parent_Pos + 1; child_Pos < opinionsPerNodeList.size(); child_Pos++) {
			childNode = opinionsPerNodeList.get(child_Pos).getNode();
			if (childNode.getParent().toString().matches(parentNode.toString())) {
				if (!childNode.isLeaf()) {
					removeNodeInList(childNode, child_Pos);
					child_Pos--; 

				} else {
					opinionsPerNodeList.remove(child_Pos);
					child_Pos--; 
				}
			}
		}
		opinionsPerNodeList.remove(parent_Pos);
	}

	public void removeAllNode() {
		for (int i = 1; i < opinionsPerNodeList.size(); i++) {
			if (opinionsPerNodeList.get(i).getNode().getParent().equals(root)) {
				treeModel = (DefaultTreeModel) treeStructure.getModel();
				treeModel.removeNodeFromParent(opinionsPerNodeList.get(i).getNode());
			}
			opinionsPerNodeList.remove(i);
			i--;
		}
		treeModel.reload();
	}
}
