package project.init;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import javax.swing.*;
import project.brain.component.Net_Client;
import project.brain.component.Net_Server;
import project.brain.component.Tree;
import org.jdesktop.jdic.browser.*;
import java.net.URL;

public class Layout extends JFrame implements WindowListener{

	private static final long serialVersionUID = 1L;
	private Container contentPanel = getContentPane();
	private JPanel treePanel;
	private JPanel opinionPanel;
	private JPanel chatPanel;
	public static JLabel statusLabel = new JLabel();
	private JPanel boardPanel;
	private JSplitPane splitPanel;
	private JMenuBar menuBar;
	private boolean flag_ConnectionNetwork = false;
	
	public static JTextArea chatOutput = new JTextArea();
	public static JTextField chatInput = new JTextField();
	public static JScrollPane chatScroll = new JScrollPane(chatOutput);
	public static JEditorPane opinionOutputArea = new JEditorPane();
	public static JScrollPane outputScroll = new JScrollPane( opinionOutputArea );
	public static JTextArea opinionInputArea = new JTextArea();
	public static JButton opinionInputButton = new JButton("Input");
	public static ImageButton webSearchButton = new ImageButton("images/search_button_blue.png");
	
    private static JMenuItem menuItemFileOpen;
    private static JMenuItem menuItemFileSave;
	
    private JMenu menuBrain;
    private static JMenuItem menuItemServer;
	private static JMenuItem menuItemClient;
	private static JMenuItem menuItemConnectionList;
	private static JMenuItem menuItemDisconnect;
	
	private JMenu menuMind;
	private static JMenuItem menuItemMindConnect;
	private static WebBrowser browser;
	
	private boolean isBrainActive = false;
	
	private JMenu menuOption;
	private Net_Client client;
	private Tree tree = new Tree();
	private boolean bOpened = false;
	

	public Layout(String title) {
		super(title);
		contentPanel.setLayout(new BorderLayout());
		
		contentPanel.add(new IntroScreen());

		menuBar = new JMenuBar();
		JMenuItem menuItem;

		JMenu menu = new JMenu("File");
		menuItemFileOpen = new JMenuItem("Open");
		menuItemFileOpen.addActionListener(new FileOpenAction());
		menuItemFileOpen.setEnabled(false);
		menu.add(menuItemFileOpen);
		menuItemFileSave = new JMenuItem("Save");
		menuItemFileSave.addActionListener(new FileSaveAction());
		menuItemFileSave.setEnabled(false);
		menu.add(menuItemFileSave);
		menu.addSeparator();
		menuItem = new JMenuItem("Exit");
		menuItem.addActionListener(new ExitAction());
		menu.add(menuItem);
		menuBar.add(menu);
		setJMenuBar(menuBar);

		menu = new JMenu("Option");
		menuBrain = new JMenu("Brainstorming");
		menuItemServer = new JMenuItem("Server");
		menuItemServer.addActionListener(new ServerAction());
		menuBrain.add(menuItemServer);
		menuItemClient = new JMenuItem("Client");
		menuItemClient.addActionListener(new ClientAction());
		menuBrain.add(menuItemClient);
		menuBrain.addSeparator();
		menuItemConnectionList = new JMenuItem("ConnectionList");
		menuItemConnectionList.setEnabled(false);
		menuItemConnectionList.addActionListener(new ConnectionListAction());
		menuBrain.add(menuItemConnectionList);
		menuBrain.addSeparator();
		menuItemDisconnect = new JMenuItem("Disconnect");
		menuItemDisconnect.setEnabled(false);
		menuItemDisconnect.addActionListener(new DisconnectAction());
		menuBrain.add(menuItemDisconnect);
		menuOption = new JMenu("Option");
		menuOption.add(menuBrain);
		menuOption.addSeparator();
		
		menuMind = new JMenu("Mindstorming");
		menuItemMindConnect = new JMenuItem("Connect");
		menuItemMindConnect.addActionListener(new MindConnectAction());
		menuMind.add(menuItemMindConnect);
		menuOption.add(menuMind);
		
		menuBar.add(menuOption);

		menu = new JMenu("Info.");
		menuItem = new JMenuItem("Usage");
		menuItem.addActionListener(new UsageAction());
		menu.add(menuItem);
		menuItem = new JMenuItem("About Program");
		menuItem.addActionListener(new AboutProgramAction());
		menu.add(menuItem);
		menuBar.add(menu);
		
		this.addWindowListener(this);
	}
		
	public void initView_Brain(){
		isBrainActive = true;
		setupOpinion();
		setupTree();
		setupChat();
		
		splitPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		JSplitPane splitPanel1 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPanel1.setLeftComponent(treePanel);
		splitPanel1.setRightComponent(opinionPanel);
		
		splitPanel.setLeftComponent(splitPanel1);
		splitPanel.setRightComponent(chatPanel);

		splitPanel1.setOneTouchExpandable(true);
		splitPanel.setOneTouchExpandable(true);
		
		if (client.getConnectFlag()) {
			contentPanel.removeAll();
			contentPanel.repaint(); 
			contentPanel.add(splitPanel, BorderLayout.CENTER);
			contentPanel.validate();
			flag_ConnectionNetwork = true;
			setNetworkMenuEnabled(true);
		}
	}
	

	public void initView_Mind(){
		browser = new WebBrowser();
		try{
			browser.setURL(new URL("http://localhost:8080/MindstormingServer/index.jsp"));
		}catch(Exception e){
			e.printStackTrace();
		}
		
		boardPanel = new JPanel(new BorderLayout());
		boardPanel.add(browser);
		
		contentPanel.removeAll();
		contentPanel.repaint(); 
		contentPanel.add(boardPanel, BorderLayout.CENTER);
		contentPanel.add(statusLabel, BorderLayout.SOUTH);
		contentPanel.validate();
		flag_ConnectionNetwork = true;
	}
	
	public static void setNetworkMenuEnabled(boolean bConnection){

		menuItemFileOpen.setEnabled(bConnection);
		menuItemFileSave.setEnabled(bConnection);
		menuItemServer.setEnabled(!bConnection);
		menuItemClient.setEnabled(!bConnection);
		menuItemDisconnect.setEnabled(bConnection);
		menuItemConnectionList.setEnabled(bConnection);
	}
	public void setFileOpenMenuEnabled(boolean bConnection){
		menuItemFileOpen.setEnabled(bConnection);
	}
	
	private void setupTree() {
		treePanel = new JPanel(new BorderLayout());
		treePanel.add(tree.toolBar, BorderLayout.NORTH);
		treePanel.add(tree.getTreeStructure(), BorderLayout.SOUTH);
		treePanel.add(new JScrollPane(tree.getTreeStructure()));
	}

	private void setupOpinion() {
		JSplitPane splitPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

		JScrollPane inputScroll = new JScrollPane(opinionInputArea);
		JPanel inputPanel = new JPanel(new BorderLayout());
		inputPanel.add(opinionInputButton, BorderLayout.EAST);
		inputPanel.add(inputScroll, BorderLayout.CENTER);
		
		JPanel iconPanel = new JPanel(new BorderLayout());
		webSearchButton.setRolloverIcon(new ImageIcon("images/search_button_green.png"));
		webSearchButton.setToolTipText("ͳ ˻ϱ");
		iconPanel.add(webSearchButton, BorderLayout.WEST);
		inputPanel.add(iconPanel, BorderLayout.NORTH);
		
		opinionInputArea.setLineWrap(true);

		opinionOutputArea.setEditable(false);
		opinionOutputArea.setContentType("text/html");
		
		JFrame jFrame = new JFrame();
		Container outputPanel = jFrame.getContentPane();
        outputPanel.setLayout( new GridBagLayout() );
		outputPanel.add(outputScroll, new GridBagConstraints(0, 0, 1, 1, 100.0, 100.0,
                        GridBagConstraints.BELOW_BASELINE, GridBagConstraints.BOTH,
                        new Insets( 1, 1, 1, 1), 0,0));
		 
		splitPanel.setTopComponent(outputPanel);
		splitPanel.setBottomComponent(inputPanel);
		splitPanel.setResizeWeight(0.9);

		opinionPanel = new JPanel(new BorderLayout());
		opinionPanel.add(new JLabel("Opinion - ȸ  ̵, ٽɿ ."),BorderLayout.NORTH);
		opinionPanel.add(splitPanel);
	}
	
	private void setupChat() {
		chatOutput.setEditable(false);

		chatScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		chatOutput.setLineWrap(true);

		chatPanel = new JPanel(new BorderLayout());
		chatPanel.add(new JLabel("Chatting - ǰ߱ȯ, ϻ ȭ ."),BorderLayout.NORTH);
		chatPanel.add(chatScroll, BorderLayout.CENTER);
		chatPanel.add(chatInput, BorderLayout.SOUTH);
	}
	
	class ServerAction implements ActionListener{
		
		private JFrame netFrame;
		private JTextField portField;
		private JTextField userField;
		private JComboBox cntAttendeeCombo;
		private JCheckBox passwordCheck;
		private JPasswordField passwordField;
		private int attendeeNumber=0;
		
		public void actionPerformed(ActionEvent e){
			if(bOpened){
				JOptionPane.showConfirmDialog(new JFrame(), "Էâ ̹  ֽϴ.",
						"˸", JOptionPane.DEFAULT_OPTION);
				return;
			}
			bOpened = true;
			netFrame = new JFrame("Է");
			netFrame.setLayout(new GridLayout(5, 2));
			
			JLabel portLabel = new JLabel("Port");
			JLabel userLabel = new JLabel("User Name");
			JLabel cntAttendeeLabel = new JLabel("Attendee Number");
			portField = new JTextField(13);
			userField = new JTextField(13);
			cntAttendeeCombo = new JComboBox();
			cntAttendeeCombo.setMaximumRowCount(4);
			passwordField = new JPasswordField(13);
			passwordField.setEnabled(true);
			passwordCheck = new JCheckBox("password");
			passwordCheck.setSelected(true);
			passwordCheck.addItemListener(new ItemListener() {
				
				@Override
				public void itemStateChanged(ItemEvent e) {
					if(e.getStateChange() == ItemEvent.SELECTED){
						passwordField.setEnabled(true);
					}
					else
						passwordField.setEnabled(false);
				}
			});
			
			cntAttendeeCombo.addItem("0.Ѿ");
			cntAttendeeCombo.addItem("1.ȥ ϱ");
			cntAttendeeCombo.addItem("2. ");
			cntAttendeeCombo.addItem("3. ");
			cntAttendeeCombo.addItem("4. ");
			cntAttendeeCombo.addItem("5.ټ ");
			cntAttendeeCombo.addItem("6. ");
			cntAttendeeCombo.addItem("7.ϰ ");
			cntAttendeeCombo.addItem("8. ");
			cntAttendeeCombo.addItem("9.ȩ ");
			cntAttendeeCombo.addItem("10. ");
			cntAttendeeCombo.addItemListener(new ItemListener() {
				@Override
				public void itemStateChanged(ItemEvent e) {
					if(e.getStateChange() == ItemEvent.SELECTED){
						attendeeNumber = Integer.parseInt(e.getItem().toString().substring(0, e.getItem().toString().indexOf('.')));
					}
				}
			});
			
			portField.setText("2222");
			userField.setText("server");
			
			JButton confirmButton = new JButton("Confirm");
			JButton cancelButton = new JButton("Cancel");
			netFrame.add(portLabel);
			netFrame.add(portField);
			netFrame.add(userLabel);
			netFrame.add(userField);
			netFrame.add(cntAttendeeLabel);
			netFrame.add(cntAttendeeCombo);
			netFrame.add(passwordCheck);
			netFrame.add(passwordField);
			netFrame.add(confirmButton);
			netFrame.add(cancelButton);
			netFrame.setSize(270, 150);
			netFrame.addWindowListener(new WindowListener() {
				@Override
				public void windowClosing(WindowEvent e) {
					bOpened=false;
				}
				public void windowOpened(WindowEvent e) {}
				public void windowIconified(WindowEvent e) {}
				public void windowDeiconified(WindowEvent e) {}
				public void windowDeactivated(WindowEvent e) {}
				public void windowClosed(WindowEvent e) {}
				public void windowActivated(WindowEvent e) {}
			});
			netFrame.setVisible(true);
			
			portField.addKeyListener(new KeyAdapter(){
				public void keyTyped(KeyEvent e){
					char c = e.getKeyChar();
					if((c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE))
						return;
					if(!(Character.isDigit(c)))
						e.consume();
				}
			});
			
			confirmButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int port = Integer.parseInt(portField.getText());
					String userName = userField.getText();
					String password = null;
					if(userName.trim().equals("")){
						JOptionPane.showConfirmDialog(new JFrame(), "̸ Էϼ.",
								"Connection Error", JOptionPane.DEFAULT_OPTION);
						return;
					}
					if(passwordCheck.isSelected()){
						password = String.copyValueOf(passwordField.getPassword());
						if(password.trim().equals("")){
							JOptionPane.showConfirmDialog(new JFrame(), "н带 Էϼ.\nн带  , üũڽ ϼ.",
									"Connection Error", JOptionPane.DEFAULT_OPTION);
							return;
						}
						new Net_Server(port, attendeeNumber, password);
					}
					else
						new Net_Server(port, attendeeNumber);

					client = new Net_Client("localhost", port, userName, password, tree, "Server");
					
					initView_Brain();
					netFrame.setVisible(false);
					bOpened = false;
				}
			});
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					netFrame.setVisible(false);
					bOpened = false;
				}
			});
		}
	}


	class ClientAction implements ActionListener {
		private JFrame netFrame;
		private JTextField portField;
		private JTextField userField;
		private JTextField addressField;
		private JPasswordField passwordField;
		public void actionPerformed(ActionEvent e){
			if(bOpened){
				JOptionPane.showConfirmDialog(new JFrame(), "Էâ ̹  ֽϴ.",
						"˸", JOptionPane.DEFAULT_OPTION);
				return;
			}
			bOpened = true;
			netFrame = new JFrame("Է");
			netFrame.setLayout(new GridLayout(5, 2));

			JLabel addressLabel = new JLabel("IP");
			JLabel portLabel = new JLabel("Port");
			JLabel userLabel = new JLabel("User Name");
			JLabel passwordLabel = new JLabel("Password");
			addressField = new JTextField(13);
			portField = new JTextField(13);
			userField = new JTextField(13);
			passwordField = new JPasswordField(13);
			
			addressField.setText("127.0.0.1");
			portField.setText("2222");
			userField.setText("client");
			
			
			JButton confirmButton = new JButton("Confirm");
			JButton cancelButton = new JButton("Cancel");
			netFrame.add(addressLabel);
			netFrame.add(addressField);
			netFrame.add(portLabel);
			netFrame.add(portField);
			netFrame.add(userLabel);
			netFrame.add(userField);
			netFrame.add(passwordLabel);
			netFrame.add(passwordField);
			netFrame.add(confirmButton);
			netFrame.add(cancelButton);
			netFrame.setSize(270, 150);
			netFrame.addWindowListener(new WindowListener() {
				@Override
				public void windowClosing(WindowEvent e) {
					bOpened = false;
				}
				public void windowOpened(WindowEvent e) {}
				public void windowIconified(WindowEvent e) {}
				public void windowDeiconified(WindowEvent e) {}
				public void windowDeactivated(WindowEvent e) {}
				public void windowClosed(WindowEvent e) {}
				public void windowActivated(WindowEvent e) {}
			});
			netFrame.setVisible(true);
			
			portField.addKeyListener(new KeyAdapter(){
				public void keyTyped(KeyEvent e){
					char c = e.getKeyChar();
					if((c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE)){
						return;
					}
					if(!(Character.isDigit(c))){
						e.consume();
					}
				}
			});
			confirmButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String ipAddr = addressField.getText();
					int port = Integer.parseInt(portField.getText());
					String user = userField.getText();
					String password = String.copyValueOf(passwordField.getPassword());
					if(user.trim().equals("")){
						JOptionPane.showConfirmDialog(new JFrame(), "̸ Էϼ.",
								"Connection Error", JOptionPane.DEFAULT_OPTION);
						return;
					}
					
					client = new Net_Client(ipAddr, port, user, password.trim(), tree, "Client");

					if(client.getConnectFlag()){
						initView_Brain();
					}
					netFrame.setVisible(false);
					bOpened = false;
				}
			});
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					netFrame.setVisible(false);
					bOpened = false;
				}
			});
		}
	}
	
	class MindConnectAction implements ActionListener{
		public void actionPerformed(ActionEvent e){
			initView_Mind();
		}
	}

	class FileOpenAction extends JFrame implements ActionListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public void actionPerformed(ActionEvent e){
			if (!flag_ConnectionNetwork) {
				JOptionPane.showConfirmDialog(this, "Ʈũ   ʾҽϴ.", "Open"
						+ " ", JOptionPane.DEFAULT_OPTION);
				return;
			}
			
			if(tree.getOpinionsPerNodeList().size()>1){
				int flag_Result = 0; 
				flag_Result = JOptionPane.showConfirmDialog(this,
						" ۾  Ͻðڴϱ?\n"
								+ "(yes : ϰ Ͽ, no :    )",
						"BrainStorming", JOptionPane.YES_NO_OPTION);
				if(flag_Result == 0){
					FileIO file = new FileIO(tree);
					file.saveFile();
				}
			}
			FileIO file = new FileIO(tree);
			if(file.openFile() == true){
				try {
					client.openedFile();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			}
		}
	}

	class FileSaveAction extends JFrame implements ActionListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public void actionPerformed(ActionEvent e){
			if (!flag_ConnectionNetwork) {
				JOptionPane.showConfirmDialog(this, "Ʈũ   ʾҽϴ.", "Save"
						+ " ", JOptionPane.DEFAULT_OPTION);
				return;
			}
			FileIO file = new FileIO(tree);
			file.saveFile();

		}
	}

	class ExitAction extends JFrame implements ActionListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public void actionPerformed(ActionEvent e){
			if (tree.getOpinionsPerNodeList().size() > 1) {
				int flag_Result = JOptionPane.showConfirmDialog(this,
						" ϰ  Ͻðڽϱ?\n"
								+ "(yes : , no : ׳ , cancel : Exit)",
						"BrainStorming", JOptionPane.YES_NO_CANCEL_OPTION);
				switch (flag_Result) {
				case 0:
					FileIO file = new FileIO(tree);
					file.saveFile();
					client.closeSocket();
					System.exit(0);
					break;
				case 1:
					client.closeSocket();
					System.exit(0);
					break;
				case 2:
					break;
				}
			} else{
				if(flag_ConnectionNetwork){
					client.closeSocket();
				}
				System.exit(0);
			}
		}
	}


	class ConnectionListAction  implements ActionListener {
		public void actionPerformed(ActionEvent e){
			client.requestConnectionInfo();
		}
	}

	class DisconnectAction implements ActionListener {
		public void actionPerformed(ActionEvent e){
			client.closeSocket();
			JOptionPane.showConfirmDialog(new JFrame(), "\nƮũ  Ǿϴ.\n Ǹ  ߰//  ǰԷ, äõ    ϴ.\n\n",
					" ", JOptionPane.DEFAULT_OPTION);
			setNetworkMenuEnabled(false);
			chatOutput.append("Ʈũ  Ǿϴ.\n\n Ǹ  ߰//  ǰԷ, äõ    ϴ.\n");
		}
	}
	
	public void windowClosing(WindowEvent e) { 
		if(isBrainActive){
			if (tree.getOpinionsPerNodeList().size() > 1) {
				int flag_Result = JOptionPane.showConfirmDialog(this,
						" ϰ  Ͻðڽϱ?\n"
								+ "(yes : , no : ׳ )",
						"BrainStorming", JOptionPane.YES_NO_OPTION);
				switch (flag_Result) {
				case 0:
					FileIO file = new FileIO(tree);
					file.saveFile();
					client.closeSocket();
					System.exit(0);
					break;
				case 1:
					client.closeSocket();
					System.exit(0);
					break;
				}
			} 
			else{
				if(flag_ConnectionNetwork){
					client.closeSocket();
				}
				System.exit(0);
			}
		}
	}
    public void windowActivated(WindowEvent e) {} 
    public void windowClosed(WindowEvent e) {} 
    public void windowDeactivated(WindowEvent e) {} 
    public void windowDeiconified(WindowEvent e) {} 
    public void windowIconified(WindowEvent e) {} 
    public void windowOpened(WindowEvent e) {} 
 

	class UsageAction extends JFrame implements ActionListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public void actionPerformed(ActionEvent e){
			String info=
			"극ν 뼳\n"+				
			"1. Ʈũ   : ޴ [Option]-[Brainstorming]\n"+
			"  1-1.  : (ȸ ü)  Ѵ.\n"+
			"    1-1-1. Port : Ʈȣ() ԷѴ.\n"+ 
			"    1-1-2. User Name : ̸ ԷѴ.\n"+
			"    1-1-3. Attentdee Number : ڽ  ȸ   Ѵ.\n"+
			"    1-1-4. Password : Ŭ̾Ʈ ӽ Է йȣ Ѵ.\n"+
			"                                   üũڽ Ѵ.\n\n"+
			" 1-2. Ŭ̾Ʈ\n"+
			"    1-2-1. IP :  IPּҸ ԷѴ.\n"+
			"    1-2-2. Port :   Ʈȣ() ԷѴ.\n"+
			"    1-2-3. User Name : ̸ ԷѴ.\n"+
			"    1-2-4. Password :   йȣ ԷѴ.\n\n"+
			"2.  ̽\n"+
			"  2-1. Ʈ() : /  ȮѴ.\n"+
			"    2-1-1. ߰ : ȸǳ뿡    .\n"+
			"    2-1-2.  : ʿ  Ѵ. θ , ڽĳ ȴ.\n"+
			"  2-2. ǰ(߾) :  忡  ǰ Ӱ ۼѴ.\n"+
			"  2-3. ä() : ȸ  ƴ ο ǻ Ѵ.\n\n"+
			"3. ϰ\n"+
			"  3-1.  : *.bsȮڸ  Ϸ ȴ.\n"+
			"  3-2. Ϻҷ : *.bsȮڸ   ҷ´.\n\n"+
			"4. \n"+
			"  4-1.  : [Info.]-[Usage] ȮѴ.\n"+
			"  4-2. α׷ : [Info.]-[About Program] ȮѴ.";
	
			JOptionPane.showMessageDialog(this, info, "Usage",
			JOptionPane.INFORMATION_MESSAGE);
		}
	}
	class AboutProgramAction implements ActionListener {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public void actionPerformed(ActionEvent e) {
			try {
				String info = "α׷ \n\n" + "ǰ  :  극ν\n\n"
						+ "ǰ  : ¶  극ν ȸǹ \n\n"
						+ "version : 1.0(2010.11.09)\n\n"
						+ "mail : callingsong@gmail.com";
				JOptionPane.showMessageDialog(new JFrame(), info, "About Program",
						JOptionPane.INFORMATION_MESSAGE);
			} catch (Exception ie) {
				JOptionPane.showConfirmDialog(new JFrame(), ie,
						"AboutProgramAction Error", JOptionPane.DEFAULT_OPTION);
			}
		}
	}
}
